# -*- coding: utf8 -*-
#  Duplicate Line plugin
#
#  spetial thanks to Micah:
#  http://www.micahcarrick.com/writing-plugins-for-gedit-3-in-python.html
#
#  Copyright (C) 2011 Nixahn <nixahn@gmail.com>
#  Copyright (C) 2007 Shaddy Zeineddine <shaddyz@users.sourceforge.net>
#  Copyright (C) 2005 Marcus Lunzenauer <mlunzena@uos.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330,
#  Boston, MA 02111-1307, USA.

from gi.repository import GObject, Gtk, Gdk, Gedit

dup_line_xml = """
<ui>
	<menubar name="MenuBar">
		<menu name="EditMenu" action="Edit">
			<placeholder name="EditOps_3">
				<separator name="DuplicateLineSep1"/>
				<menuitem name="DeleteLine"          action="DeleteLine"/>
				<menuitem name="DuplicateLine"       action="DuplicateLine"/>
			</placeholder>
		</menu>
	</menubar>
</ui>
"""

class DuplicateLine(GObject.Object, Gedit.WindowActivatable):
	__gtype_name__ = "DuplicateLine"
	window = GObject.property(type=Gedit.Window)

	def __init__(self):
		GObject.Object.__init__(self)

	def delete_line(self, action):
		view = self.window.get_active_view()
		# GTK_DELETE_PARAGRAPH_ENDS=5
		view.do_delete_from_cursor(5,1)

	def duplicate_line(self, action):
		view = self.window.get_active_view()
		doc	= self.window.get_active_document()
		doc.begin_user_action()
		itorig=doc.get_iter_at_mark(doc.get_insert())
		offset=itorig.get_line_offset()
		# GTK_MOVEMENT_PARAGRAPH_ENDS=6
		if offset:
			view.do_move_cursor(view,6,-1,0)
		itstart = doc.get_iter_at_mark(doc.get_insert())
		itend = doc.get_iter_at_mark(doc.get_insert())
		itend.forward_line();
		line = doc.get_slice(itstart, itend, True)
		doc.insert_at_cursor(line)
		# GTK_MOVEMENT_PARAGRAPHS=5		
		# view.do_move_cursor(view,5,-1,0)
		# GTK_MOVEMENT_DISPLAY_LINE_ENDS = 4
		# view.do_move_cursor(view,4,1,0)
		# GTK_MOVEMENT_LOGICAL_POSITIONS = 0
		if offset:
			view.do_move_cursor(view,0,offset,0)
		doc.end_user_action()

	def do_activate(self):
		actions = [
			('DeleteLine',          None, 'Delete To End Of Line', '<Shift><Control>d', "Delete To End Of Line", self.delete_line),
			('DuplicateLine',       None, 'Duplicate Line',        '<Control>j',        "Duplicate Line",        self.duplicate_line)
		]

		self._actions = Gtk.ActionGroup("GeditDuplicateLinePluginActions")
		self._actions.add_actions(actions)

		manager = self.window.get_ui_manager()
		manager.insert_action_group(self._actions)

		self._ui_merge_id = manager.add_ui_from_string(dup_line_xml)
		manager.ensure_update()

	def do_deactivate(self):
		manager = self.window.get_ui_manager()
		manager.remove_ui(self._ui_merge_id)
		manager.remove_action_group(self._actions)
		manager.ensure_update()
		
	def do_update_state(self):
		view = self.window.get_active_view()
		self._actions.set_sensitive(bool(view and view.get_editable()))
